"""
heliopy configuration utility
"""
import configparser
import os
import pymms
from pathlib import Path


def get_config_file():
    """
    Find the configuration file. Locations are{}

    1. ~/.pymmsrc/pymmsrc
    2. <installation folder>/config.ini
    3. <installation folder>/template_config.ini

    Returns
    -------
    config_file : str
        Filepath of the ``pymms`` configuration file
    """
    # Get user configuration location
    config_files = [Path('~/.pymmsrc/pymmsrc').expanduser(),
                    Path(pymms.__file__).parent / 'config.ini',
                    Path(pymms.__file__).parent / 'config_template.ini']
    
    for f in config_files:
        if f.is_file():
            return str(f)


def load_config():
    """
    Read the configuration file. Locations are:

    1. <installation folder>/template_config.ini
    2. <installation folder>/config.ini

    Returns
    -------
    config : dict
        Dictionary containing configuration options.
    """    
    config_location = get_config_file()
    config = configparser.ConfigParser()
    config.read(config_location)
    config_dict = {}
    
    # Set data download directory
    for dirname in ('data_root', 'dropbox_root', 'mirror_root'):
        root = os.path.expanduser(config['DIRS'][dirname])
        if root == 'None':
            config_dict[dirname] = None
            continue
        
        # Make any default directories system-independent
        if os.name == 'nt':
            root = root.replace('/', '\\')
        config_dict[dirname] = root
        
        # Create data download if not created
        if not os.path.isdir(root):
            print('Creating root data directory {}'.format(root))
            os.makedirs(root)

    # login credentials
    for cred in ('username', 'password'):
        config_dict[cred] = config['SDCLOGIN'][cred]
        if config_dict[cred] == 'None':
            config_dict[cred] = None

    config_dict['gls_root'] = config['GLS']['gls_root']
    if config_dict['gls_root'] == 'None':
        config_dict['gls_root'] = None

    return config_dict



""" Module to convert between TAI and UTC time scales.

    utc2tai(datetime.datetime.now())
        returns a datetime.datetime object representing TAI from a datetime.datetime
        object at UTC.
    tai2utc(datetime(1992, 6, 2, 8, 7, 9))
        returns a datetime.datetime object representing UTC from a datetime.datetime
        object at TAI.
    taisec2utc(1400000000.0)
        returns a datetime.datetime object representing TAI from a real value
        representing seconds since the TAI epoch
    

    Originally from:
        https://pypi.python.org/pypi/tai64n
        modifications:
            add 2012+ leapseconds
            add taisec2utc, utc2taisec
            fix tai2utc for times near a leapsecond
            remove hex string decoding
        @author: Kim Kokkonen
"""


from datetime import timedelta, datetime
from operator import itemgetter
import time
import pytz

def utc_datetime(year, month, day, hour=0, minute=0, second=0, microsecond=0, tzinfo=None):
    '''Returns a datetime with a UTC timezone'''
    dt = datetime(year, month, day, hour, minute, second, microsecond, tzinfo=tzinfo)
    dt = to_utc_tz(dt)
    return dt


def to_utc_tz(dt):
    '''Return a datetime properly converted to the UTC timezone.'''
    if str(dt.tzinfo) == 'UTC':
        return dt

    if dt.tzinfo is None:
        dt = dt.replace(tzinfo=pytz.UTC)
    dt = dt.astimezone(pytz.UTC)
    return dt


# offset in seconds between unix and TAI epochs
unix_epoch = utc_datetime(1970, 1, 1)
tai_unix_offset = (unix_epoch - utc_datetime(1958, 1, 1)).total_seconds()


def __conversion_table():
    """ returns [datetime, value] ordered reverse by date
        where value == seconds between TAI and UTC

    Example:
    >>> __conversion_table()[0]
    (datetime.datetime(1972, 1, 1, 0, 0), 10.0)
    """
    # update this table as new values become known
    # source: ftp://maia.usno.navy.mil/ser7/tai-utc.dat
    conversion_table = [(utc_datetime(1972, 0o1, 1), 10.0),
                        (utc_datetime(1972,0o7, 1), 11.0),
                        (utc_datetime(1973,0o1, 1), 12.0),
                        (utc_datetime(1974,0o1, 1), 13.0),
                        (utc_datetime(1975,0o1, 1), 14.0),
                        (utc_datetime(1976,0o1, 1), 15.0),
                        (utc_datetime(1977,0o1, 1), 16.0),
                        (utc_datetime(1978,0o1, 1), 17.0),
                        (utc_datetime(1979,0o1, 1), 18.0),
                        (utc_datetime(1980,0o1, 1), 19.0),
                        (utc_datetime(1981,0o7, 1), 20.0),
                        (utc_datetime(1982,0o7, 1), 21.0),
                        (utc_datetime(1983,0o7, 1), 22.0),
                        (utc_datetime(1985,0o7, 1), 23.0),
                        (utc_datetime(1988,0o1, 1), 24.0),
                        (utc_datetime(1990,0o1, 1), 25.0),
                        (utc_datetime(1991,0o1, 1), 26.0),
                        (utc_datetime(1992,0o7, 1), 27.0),
                        (utc_datetime(1993,0o7, 1), 28.0),
                        (utc_datetime(1994,0o7, 1), 29.0),
                        (utc_datetime(1996,0o1, 1), 30.0),
                        (utc_datetime(1997,0o7, 1), 31.0),
                        (utc_datetime(1999,0o1, 1), 32.0),
                        (utc_datetime(2006,0o1, 1), 33.0),
                        (utc_datetime(2009,0o1, 1), 34.0),
                        (utc_datetime(2012,0o7, 1), 35.0),
                        (utc_datetime(2015,0o7, 1), 36.0),
                        (utc_datetime(2017,0o1, 1), 37.0),
                        # add new values here
                        ]
    conversion_table.sort(key=itemgetter(0), reverse=True)
    return conversion_table


def __tai_seconds(date, table=None):
    """ returns seconds of TAI-offset from UTC at date given.
        Works only on dates later than0o1.01.1972.

    Example:
        >>> __tai_seconds(utc_datetime(1992, 6, 2, 8, 7, 9))
        26.0
        >>> __tai_seconds(utc_datetime(1971, 6, 2, 8, 7, 9))
        False
    """

    # Avoids risk of conversion table being changed
    if table is None:
        table = __conversion_table()

    for x in table:
        if date > x[0]:
            return x[1]
    return False


def tai2utc(date):
    """ converts datetime.datetime TAI to datetime.datetime UTC.
        Works only on dates later than0o1.01.1972.

    Example
        >>> tai2utc(utc_datetime(1992, 6, 2, 8, 7, 9))
        datetime.datetime(1992, 6, 2, 8, 6, 43)
    """
    # leapseconds at the tai time
    seconds = __tai_seconds(date)
    # first approximation to utc datetime
    dateutc = date - timedelta(0, seconds)
    # leap seconds at corresponding utc datetime
    secondsu = __tai_seconds(dateutc)

    if seconds > secondsu:
        # utc and tai on different sides of a leapsecond
        if date.second < secondsu:
            # tai time before the new leapsecond is applied
            return seconds and (date - timedelta(0, secondsu))
        # new leapsecond is applied
        # datetime can't represent 23:59:60 so return 23:59:59 again
        return secondsu and dateutc
    # no leapsecond shift involved
    return seconds and dateutc


def utc2tai(date):
    """ converts datetime.datetime UTC to datetime.datetime TAI.
        Works only on dates later than0o1.01.1972.

    Example
        >>> utc2tai(datetime(1992, 6, 2, 8, 6, 43))
        datetime.datetime(1992, 6, 2, 8, 7, 9)
    """
    seconds = __tai_seconds(to_utc_tz(date))
    return seconds and (date + timedelta(0, seconds))


def utc2taisec(date):
    """ converts datetime.datetime UTC to TAI seconds since the 1958 epoch.
        Works only on dates later than0o1.01.1972.

    Example
        t = utc_datetime(2015, 9, 1, 17, 45, 0)
        t1=tai.utc2taisec(t)
        t1
        Out[6]: 1819820736.0
        tai.taisec2utc(t1)
        Out[7]: datetime.datetime(2015, 9, 1, 17, 45, tzinfo=<UTC>)
    """
    # note this implies date must already be UTC-tz-aware
    seconds = __tai_seconds(date.replace(tzinfo=pytz.UTC))
    return seconds and (date - unix_epoch).total_seconds() + seconds + tai_unix_offset


def taisec2utc(tai):
    """ returns a datetime.datetime (UTC) object from a tai time in seconds.
        Works only on dates after0o1.01.1972.
        If the tai time includes fractional seconds, the fractional part is
        carried straight across into the python datetime microseconds field.

        Args
            tai: TAI time in integer seconds since 1958-01-01.

        Returns
            a datetime.datetime object at UTC

        Example
        >>> taisec2utc(1400000000.0)
        datetime.datetime(2002, 5, 13, 16, 52, 48)
    """
    st = time.gmtime(tai - tai_unix_offset)
    micro = min(999999, int(round(1000000 * (tai % 1))))
    dt = utc_datetime(st.tm_year, st.tm_mon, st.tm_mday,
                                     st.tm_hour, st.tm_min, st.tm_sec, micro)
    return tai2utc(dt)


if __name__ == '__main__':
    import doctest
    doctest.testmod()




import numpy as np
import datetime as dt
import spacepy
from spacepy import pycdf
import pandas as pd
from pandas import DataFrame, Series
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import os.path
import pymms
from pymms import mms_utils
import pdb
import sqlite3
from datetime import datetime
from datetime import timedelta

## Creating the pymms object
# First, we create an instance of the object that communicates with the SDC. For the sake of this
# example, we will start with data from `2015-10-16` because there are several magnetopause crossings
# and a well-studied electron diffusion region event. Also, for simplicity we will work with data from
# the MMS1 spacecraft. Data from other spacecraft can be loaded by changing the `sc` property
# below to `'mms2'`, `'mms3'`, or `'mms4'`.

# Create an instance of SDC object
sdc = pymms.MrMMS_SDC_API()

# Declare the vertical user who's folder in /data/ will be store the downloaded MMS data
data_downloaduser = 'colin'

# Declare the vertical user who's folder will be store the SQL DB
data_downloaduser = 'colin'

def data_export(spacecraft, level, start_date, end_date):
    # Define the spacecraft. We will use the variable later when accessing the CDF files.
    sc = spacecraft
    #level = 'sitl'                    # 'l2' or 'sitl'
    level = level
    start_date = start_date.strftime('%Y-%m-%dT%H:%M:%S')
    end_date = end_date.strftime('%Y-%m-%dT%H:%M:%S')
    data_root = os.path.expanduser('~/data/{}/mms/').format(data_downloaduser)
            # Specifying data_root="~/" does not expand the tilde yet
            # However, if data_root=None, then ~/data is the default
    
    # Set attributes
    sdc.sc = sc
    sdc.start_date = start_date
    sdc.end_date = end_date
    sdc.data_root = data_root
    
    ## Working with CDF Files
    # CDF files are somewhat like netCDF or HDF5 files in that the contain data as well as metadata.
    # Data is associated with variable names and variable metadata, or variable attributes. The file itself
    # has metadata in the form of global attributes. For our purpose, we are interested in determining the
    # variable names, what they mean, then selecting the subset of variables that are relevant to us. To do that,
    # we will need to download an MMS CDF data file and make use of pycdf from the spacepy package.
    
    # Downloading an MMS CDF File
    # Here, we will give a brief example of how to download a CDF file using the `pymms` package. We pick a
    # file from the fluxgate magnetometer (FGM) team containing magnetic field data. For demonstration purposes,
    # we select a science-quality data file (`level='l2'`) when the instrument was sampling in survey
    # mode (`mode='srvy'`). [Since the Geocentric Solar Ecliptic](https://sscweb.gsfc.nasa.gov/users_guide/Appendix_C.html)
    # (GSE) coordinate system is the standard for MMS data, we will extract non-scalar data in this system.
    
    
    # First, define variables, as they will be used in creating variable names
    fgm_mode = 'srvy'
    fgm_coords = 'gse'
    fgm_instr = 'fgm'
    fgm_level = 'l2'
    
    # Set object properties and get data
    sdc.instr = fgm_instr
    sdc.mode = fgm_mode
    sdc.level = fgm_level
    files = sdc.Download()
    
    print('FGM Files:')
    print(*files, sep='\n')
    
    
    ## CDF Attributes and Variables
    
    # In order to access data in a CDF file, it is necessary to know the names of the variables contained within. Often,
    # it is also important to know additional information about the file contents or variable data. This metadata
    # is contained in the global and variable attributes.
    # The most important variable attributes are CATDESC, which describes the variable, FILLVAL, which gives the
    # value used for bad or missing data, and DEPEND_[0-3], which list the dependent variables of a data variable.
    # Typically, the dimensions of CDF variables are ordered as [nRecs, nDep1, nDep2, nDep3], where nRecs is the total
    # number of records, each record having dimensions [nDep1, nDep2, nDep3]. The value of DEPEND_0 is typically 'Epoch',
    # indicating that the CDF variable 'Epoch' is a dependency. The 'Epoch' variable contains a CDF Epoch time stamp at
    # each of the nRecs number of records. Similarly DEPEND_[1-3] variables point to other variables in the CDF file
    # that act as dependencies. If you want to plot a variable, you will need to also extract its 'DEPEND_0' variables.
    
    # Variables and attributes are described in more detail in the ISTP CDF Guidelines. Below, we demonstrate
    # how to obtain attribute and variable names and values.
    
    
    # Open the file and pr
    cdf = pycdf.CDF(files[0])
    
    # Show global attribute
    print('Global Attributes:')
    for gAttrName in cdf.attrs:
        print('\t' + gAttrName)
    
    # Show variable names
    print('\nVariable Names:')
    for varName in cdf:
        print('\t' + varName)
    
    # Select the magnetic field variable
    vname = '_'.join((sc, fgm_instr, 'b', fgm_coords, fgm_mode, fgm_level))
    
    # Show variable attributes for a particular variable
    print('\nVariable Attributes for "' + vname + '":')
    for vAttrName in cdf[vname].attrs:
        print('\t' + vAttrName)
    
    # Important variable attributes:
    print('\nValues of Important Variable Attributes:')
    print('\t', 'CATDESC: ', cdf[vname].attrs['CATDESC'])
    print('\t', 'FILLVAL: ', cdf[vname].attrs['FILLVAL'])
    print('\t', 'DEPEND_0: ', cdf[vname].attrs['DEPEND_0'])
    
    
    ## FGM
    # The FGM dataset contains magnetic field data from the fluxgate magnetometer (FGM).
    # [Since the Geocentric Solar Ecliptic](https://sscweb.gsfc.nasa.gov/users_guide/Appendix_C.html) (GSE)
    # coordinate system is the standard for MMS data, we will extract non-scalar data in this system.
    
    # Download & Read Data
    # Now we can read data and its corresponding time stamps for a CDF variable. We choose the `'mms1_fgm_b_gse_srvy_l2'`
    # variable because, as shown above, its `CATDESC` attribute describes it as the magnetic field in GSE coordinates.
    # In order to be completely general, I will build the variable names from the attributes we have already defined.
    # Variable names have the convention of `sc_instr_param_coords_optdesc_mode_level`, where `param` describes the
    # quantity and `coords` is the coordinate system whenever relevant. Other components are similar to the file name
    # conventions.
    
    # Update instrument-specific variables
    fgm_mode = 'srvy'
    if level == 'sitl':
        fgm_coords = 'dmpa'
        fgm_instr = 'dfg'
        fgm_level = 'ql'
    else:
        fgm_coords = 'gse'
        fgm_instr = 'fgm'
        fgm_level = 'l2'
    
    # Set object properties
    sdc.instr = fgm_instr
    sdc.mode = fgm_mode
    sdc.level = fgm_level
    
    # Download data
    files = sdc.Download()
    files = mms_utils.sort_files(files)[0]
    
    # Read the magnetic field and its time stamps
    if level == 'l2':
        b_vname = '_'.join((sc, fgm_instr, 'b', fgm_coords, fgm_mode, fgm_level))
    else:
        b_vname = '_'.join((sc, fgm_instr, fgm_mode, fgm_coords))
    
    fgm_t = []
    fgm_b = []
    
    print('FGM Files:')
    for file in files:
        # Open the file
        cdf = pycdf.CDF(file)
    
        # Read the data
        #   - Convert numpy arrays to lists to make appending easier
        fgm_t += list(cdf[cdf[b_vname].attrs['DEPEND_0']][:])
        fgm_b += list(cdf[b_vname][:])
    
        # Close the file
        cdf.close()
        print('  ' + file)
    
    # Convert back to numpy arrays
    fgm_t = np.array(fgm_t)
    fgm_b = np.array(fgm_b)
    
    # Compute clock and normal angles
    fgm_ca = np.rad2deg(np.arctan2(fgm_b[:,1], fgm_b[:,2]))
    fgm_tbn = np.rad2deg(np.arctan2(fgm_b[:,0], fgm_b[:,2]))
    
    ## Data Frame
    
    # Create a dictionary
    fgm_data = {
        'Time' :  fgm_t,
        'Bx' : fgm_b[:,0],
        'By' : fgm_b[:,1],
        'Bz' : fgm_b[:,2],
        'Bmag' : fgm_b[:,3],
        'clock_angle' : fgm_ca,
        'normal_angle' : fgm_tbn
    }
    
    # Convert dictionary to data from
    fgm_data = pd.DataFrame(fgm_data, columns=fgm_data.keys())
    
    
    ## EDP
    # Now for electric field and spacecraft potential data from the Electric Field Double Pobles (EDP).
    
    # Download & Read
    
    # Update instrument-specific variables
    edp_instr = 'edp'
    edp_mode = 'fast'
    edp_level = level
    dce_optdesc = 'dce'
    scpot_optdesc = 'scpot'
    
    if level == 'l2':
        edp_coords = 'gse'
    else:
        edp_coords = 'dsl'
    
    # EDP variable names
    e_vname = '_'.join((sc, 'edp', dce_optdesc, edp_coords, edp_mode, edp_level))
    scpot_vname = '_'.join((sc, 'edp', scpot_optdesc, edp_mode, edp_level))
    
    # Download DCE files
    sdc.instr = edp_instr
    sdc.mode = edp_mode
    sdc.level = edp_level
    sdc.optdesc = dce_optdesc
    dce_files = sdc.Download()
    dce_files = mms_utils.sort_files(dce_files)[0]
    
    # Download SCPOT files
    sdc.optdesc = scpot_optdesc
    scpot_files = sdc.Download()
    scpot_files = mms_utils.sort_files(scpot_files)[0]
    
    # Read the data
    edp_t = []
    edp_e = []
    edp_v = []
    print('EDP Files:')
    for ifile, file in enumerate(dce_files):
        # Open the file
        dce_cdf = pycdf.CDF(dce_files[ifile])
        scpot_cdf = pycdf.CDF(scpot_files[ifile])
    
        # Read data and replace fill value with NaN
        e = dce_cdf[e_vname][:]
        v = scpot_cdf[scpot_vname][:]
        e[e == dce_cdf[e_vname].attrs['FILLVAL']] = np.nan
        v[v == scpot_cdf[scpot_vname].attrs['FILLVAL']] = np.nan
    
        # Read the data
        #   - Convert numpy arrays to lists to make appending easier
        edp_t += list(dce_cdf[dce_cdf[e_vname].attrs['DEPEND_0']][:])
        edp_e += list(e)
        edp_v += list(v)
    
        # Close the file
        dce_cdf.close()
        scpot_cdf.close()
        print('  ' + dce_files[ifile])
        print('  ' + scpot_files[ifile])
    
    # Convert back to numpy arrays
    edp_t = np.array(edp_t)
    edp_e = np.array(edp_e)
    edp_v = np.array(edp_v)
    
    
    ## Data Frame
    
    # Create a dictionary
    edp_data = {
        'Time' :  edp_t,
        'Ex' : edp_e[:,0],
        'Ey' : edp_e[:,1],
        'Ez' : edp_e[:,2],
        'scpot' : edp_v
    }
    
    # Convert dictionary to data from
    edp_data = pd.DataFrame(edp_data, columns=edp_data.keys())
    
    ## FPI
    # Next, we will repeat the process for the Fast Plasma Instrument (FPI), which consists of the Dual Electron
    # Spectrometer (DES) and the Dual Ion Spectrometer (DIS). These measure characteristics of the electron and
    # ion plasmas, respectively. Here, we are interested in the density, velocity, and temperature.
    
    # Normally, survey mode files are a combination of fast and slow survey data and span an entire day. Because FPI
    # produces so much data, however, it is only operated in fast survey mode and its "daily files" are broken up
    # into several files of shorter time intervals.
    
    
    # DIS: Download and Read
    
    # Update instrument-specific variables
    dis_instr = 'fpi'
    dis_mode = 'fast'
    
    if level == 'sitl':
        dis_coords = 'dbcs'
        dis_level = 'ql'
        dis_optdesc = 'dis'
    else:
        dis_coords = 'gse'
        dis_level = level
        dis_optdesc = 'dis-moms'
    
    # Set attributes
    sdc.instr = dis_instr
    sdc.mode = dis_mode
    sdc.level = dis_level
    sdc.optdesc = dis_optdesc
    
    # DIS variable names
    n_vname = '_'.join((sc, 'dis', 'numberdensity', dis_mode))
    v_vname = '_'.join((sc, 'dis', 'bulkv', dis_coords, dis_mode))
    t_para_vname = '_'.join((sc, 'dis', 'temppara', dis_mode))
    t_perp_vname = '_'.join((sc, 'dis', 'tempperp', dis_mode))
    espec_vname = '_'.join((sc, 'dis', 'energyspectr', 'omni', dis_mode))
    
    # Open the file
    files = sdc.Download()
    files = mms_utils.sort_files(files)[0]
    
    # Read the data
    dis_t = []
    dis_n = []
    dis_v = []
    dis_temp_para = []
    dis_temp_perp = []
    dis_espec = []
    dis_e = []
    print('DIS Files:')
    for file in files:
        # Open the file
        cdf = pycdf.CDF(file)
    
        # Read timee and shift to center of interval
        #   - There must be a bug in the CDF package because the Epoch_plus_var variables
        #     are read as empty but really contain scalar values
        t = cdf[cdf[n_vname].attrs['DEPEND_0']][:]
    #    dt_minus = t.attrs['DELTA_MINUS_VAR']
    #    dt_plus = t.attrs['DELTA_PLUS_VAR']
        dt_minus = 0
        dt_plus = 4.5
        t += dt.timedelta(seconds=(dt_plus - dt_minus) / 2.0)
    
        # Read the data
        #   - Convert numpy arrays to lists to make appending easier
        dis_t += list(t)
        dis_n += list(cdf[n_vname][:])
        dis_v += list(cdf[v_vname][:])
        dis_temp_para += list(cdf[t_para_vname][:])
        dis_temp_perp += list(cdf[t_perp_vname][:])
        dis_espec += list(cdf[espec_vname][:])
        dis_e += list(cdf[cdf[espec_vname].attrs['DEPEND_1']][:])
    
        # Close the file
        cdf.close()
        print('  ' + file)
    
    # Convert back to numpy arrays
    dis_t = np.array(dis_t)
    dis_n = np.array(dis_n)
    dis_v = np.array(dis_v)
    dis_temp_para = np.array(dis_temp_para)
    dis_temp_perp = np.array(dis_temp_perp)
    dis_espec = np.array(dis_espec)
    dis_e = np.array(dis_e)
    
    # Compute velocity magnitude
    dis_vmag = np.sqrt(dis_v[:,0]**2.0 + dis_v[:,1]**2.0 + dis_v[:,2]**2.0)
    
    # Compute scalar temperature
    dis_temp = 1.0/3.0 * (2.0*dis_temp_perp + dis_temp_para)
    
    ## Data Frame
    
    # Create a dictionary
    dis_data = pd.DataFrame()
    dis_data['Time'] = pd.Series(dis_t)
    dis_data['N'] = pd.Series(dis_t)
    dis_data['Vx'] = pd.Series(dis_t)
    dis_data['Vy'] = pd.Series(dis_t)
    dis_data['Vz'] = pd.Series(dis_t)
    dis_data['Vmag'] = pd.Series(dis_t)
    dis_data['Tpara'] = pd.Series(dis_t)
    dis_data['Tperp'] = pd.Series(dis_t)
    dis_data['T'] = pd.Series(dis_temp)
    dis_data = {
        'Time' :  dis_t,
        'N' : dis_n,
        'Vx' : dis_v[:,0],
        'Vy' : dis_v[:,1],
        'Vz' : dis_v[:,2],
        'Vmag' : dis_vmag,
        'Tpara' : dis_temp_para,
        'Tperp' : dis_temp_perp,
        'T' : dis_temp
    #    'ESpec': dis_espec,
    #    'Energy': dis_e,
    }
    
    # Convert dictionary to data from
    dis_data = pd.DataFrame(dis_data, columns=dis_data.keys())
    
    # Add dis_espec to dis_data
    for i in range(dis_espec.shape[1]):
        dis_data['ESpec_E{:02}'.format(i)] = Series(data=dis_espec[:,i])
    
    
    ## DES
    
    # Update instrument-specific variables
    des_instr = 'fpi'
    des_mode = 'fast'
    
    if level == 'sitl':
        des_coords = 'dbcs'
        des_level = 'ql'
        des_optdesc = 'des'
    else:
        des_coords = 'gse'
        des_level = level
        des_optdesc = 'des-moms'
    
    # Set attributes
    sdc.instr = des_instr
    sdc.mode = des_mode
    sdc.level = des_level
    sdc.optdesc = des_optdesc
    
    # DIS variable names
    n_vname = '_'.join((sc, 'des', 'numberdensity', des_mode))
    v_vname = '_'.join((sc, 'des', 'bulkv', des_coords, des_mode))
    t_para_vname = '_'.join((sc, 'des', 'temppara', des_mode))
    t_perp_vname = '_'.join((sc, 'des', 'tempperp', des_mode))
    espec_vname = '_'.join((sc, 'des', 'energyspectr', 'omni', des_mode))
    pad_low_vname = '_'.join((sc, 'des', 'pitchangdist', 'lowen', des_mode))
    pad_mid_vname = '_'.join((sc, 'des', 'pitchangdist', 'miden', des_mode))
    pad_high_vname = '_'.join((sc, 'des', 'pitchangdist', 'highen', des_mode))
    
    
    # Open the file
    files = sdc.Download()
    files = mms_utils.sort_files(files)[0]
    
    # Read the data
    des_t = []
    des_n = []
    des_v = []
    des_temp_para = []
    des_temp_perp = []
    des_espec = []
    des_energy = []
    des_pad_low = []
    des_pad_mid = []
    des_pad_high = []
    des_pa = []
    print('DES Files:')
    for file in files:
        # Open the file
        cdf = pycdf.CDF(file)
    
        # Read timee and shift to center of interval
        #   - There must be a bug in the CDF package because the Epoch_plus_var variables
        #     are read as empty but really contain scalar values
        t = cdf[cdf[n_vname].attrs['DEPEND_0']][:]
    #    dt_minus = t.attrs['DELTA_MINUS_VAR']
    #    dt_plus = t.attrs['DELTA_PLUS_VAR']
        dt_minus = 0
        dt_plus = 4.5
        t += dt.timedelta(seconds=(dt_plus - dt_minus) / 2.0)
    
        # Read the data
        des_t += list(t)
        des_n += list(cdf[n_vname][:])
        des_v += list(cdf[v_vname][:])
        des_temp_para += list(cdf[t_para_vname][:])
        des_temp_perp += list(cdf[t_perp_vname][:])
        des_espec += list(cdf[espec_vname][:])
        des_energy += list(cdf[cdf[espec_vname].attrs['DEPEND_1']][:])
        des_pad_low += list(cdf[pad_low_vname][:])
        des_pad_mid += list(cdf[pad_mid_vname][:])
        des_pad_high += list(cdf[pad_high_vname][:])
        des_pa += list(cdf[cdf[pad_low_vname].attrs['DEPEND_1']][:])
    
        # Close the file
        cdf.close()
        print('  ' + file)
    
    # Convert back to numpy arrays
    des_t = np.array(des_t)
    des_n = np.array(des_n)
    des_v = np.array(des_v)
    des_temp_para = np.array(des_temp_para)
    des_temp_perp = np.array(des_temp_perp)
    des_espec = np.array(des_espec)
    des_energy = np.array(des_energy)
    des_pad_low = np.array(des_pad_low)
    des_pad_mid = np.array(des_pad_mid)
    des_pad_high = np.array(des_pad_high)
    des_pa = np.array(des_pa)
    
    # Compute velocity magnitude
    des_vmag = np.sqrt(des_v[:,0]**2.0 + des_v[:,1]**2.0 + des_v[:,2]**2.0)
    
    # Compute scalar temperature
    des_temp = 1.0/3.0*(2.0*des_temp_perp + des_temp_para)
    
    # Compute pich angle distribution
    des_pad = (des_pad_low + des_pad_mid + des_pad_high) / 3.0
    
    # Create a dictionary
    des_data = {
        'Time' :  des_t,
        'N' : des_n,
        'Vx' : des_v[:,0],
        'Vy' : des_v[:,1],
        'Vz' : des_v[:,2],
        'Vmag' : des_vmag,
        'Tpara' : des_temp_para,
        'Tperp' : des_temp_perp,
        'T' : des_temp
    #    'ESpec': dis_espec,
    #    'Energy': dis_e,
    #    'PAD': des_pad,
    #    'PA': des_pa
    }
    
    # Convert dictionary to data from
    des_data = pd.DataFrame(des_data, columns=des_data.keys())
    
    # Add des_espec to des_data
    for i in range(des_espec.shape[1]):
        des_data['ESpec_E{:02}'.format(i)] = Series(data=des_espec[:,i])
    
    # EDI - Should work, but is disabled until verified
    
    # Update instrument-specific variables
    edi_instr = 'edi'
    edi_mode = 'srvy'
    edi_optdesc = None   # Get whatever is available
    
    if level == 'sitl':
        edi_level = 'ql'
    else:
        edi_level = level
    
    # Set attributes
    sdc.instr = edi_instr
    sdc.mode = edi_mode
    sdc.level = edi_level
    sdc.optdesc = 'amb'
    
    # Figure out which data product is available
    files = sdc.FileNames()
    parts = mms_utils.parse_filename(files)
    edi_optdesc = [p[4] for p in parts]
    
    # EDI variable names
    cts1_0_vname = '_'.join((sc, edi_instr, 'flux1', '0', edi_mode, 'l2'))
    cts1_180_vname = '_'.join((sc, edi_instr, 'flux1', '180', edi_mode, 'l2'))
    
    # Open the file
    files = sdc.Download()
    files = mms_utils.sort_files(files)[0]
    
    # Read the data
    edi_t = []
    edi_cts1_0 = []
    edi_cts1_180 = []
    
    print('EDI Files:')
    for file in files:
        # Open the file
        cdf = pycdf.CDF(file)
    
        # Read the datafi
        edi_t += list(cdf[cdf[cts1_0_vname].attrs['DEPEND_0']][:])
        edi_cts1_0 += list(cdf[cts1_0_vname][:])
        edi_cts1_180 += list(cdf[cts1_180_vname][:])
    
        # Close the file
        cdf.close()
        print('  ' + file)
    
    # Convert back to numpy arrays
    edi_t = np.array(edi_t)
    edi_cts1_0 = np.array(edi_cts1_0)
    edi_cts1_180 = np.array(edi_cts1_180)
    
    #TODO: This is disabled for now, because the data is not a scalar
    
    # Create a dictionary
    edi_data = {
        'Time' : edi_t,
        'cts1_0' : edi_cts1_0,
        'cts1_180' : edi_cts1_180
    }
    
    #Convert dictionary to data from
    print(type(edi_data))
    print(edi_data.keys())
    print(edi_data['Time'].shape)
    edi_data = pd.DataFrame(edi_data, columns=edi_data.keys())
    
    ## Interpolate All Values to `t_des`
    # In this step, we need to get all variables into the same time basis. We will interpolate data
    # from FGM and DIS onto the time tags of DES.
    
    # Convert datetime objects to floats
    des_t_stamp = [t.timestamp() for t in des_t]
    fgm_t_stamp = [t.timestamp() for t in fgm_t]
    dis_t_stamp = [t.timestamp() for t in dis_t]
    edp_t_stamp = [t.timestamp() for t in edp_t]
    edi_t_stamp = [t.timestamp() for t in edi_t]
    
    # Interpolate FGM data
    #   - An Nx4 array, ordered as (Bx, By, Bz, |B|)
    nTimes = len(des_t_stamp)
    nComps = np.size(fgm_b, 1)
    fgm_b_interp = np.zeros([nTimes, nComps], dtype=float)
    for idx in range(nComps):
        fgm_b_interp[:,idx] = np.interp(des_t_stamp, fgm_t_stamp, fgm_b[:,idx])
    fgm_clock_angle_interp = np.interp(des_t_stamp, fgm_t_stamp, fgm_ca)
    fgm_normal_angle_interp  = np.interp(des_t_stamp, fgm_t_stamp, fgm_tbn)
    
    # Interpolate DIS data
    dis_n_interp = np.interp(des_t_stamp, dis_t_stamp, dis_n)
    dis_temp_para_interp = np.interp(des_t_stamp, dis_t_stamp, dis_temp_para)
    dis_temp_perp_interp = np.interp(des_t_stamp, dis_t_stamp, dis_temp_perp)
    dis_temp_interp = np.interp(des_t_stamp, dis_t_stamp, dis_temp)
    # An Nx3 array, ordered as (Vx, Vy, Vz)
    nComps = np.size(dis_v, 1)
    dis_v_interp = np.zeros([nTimes, nComps])
    for idx in range(nComps):
        dis_v_interp[:,idx] = np.interp(des_t_stamp, dis_t_stamp, dis_v[:,idx])
    # An Nx32 array, ordered as (ESpec_00, ESpec_01, ... , ESpec_30, ESpec_31)
    nComps = np.size(dis_espec, 1)
    dis_espec_interp = np.zeros([nTimes, nComps])
    for idx in range(nComps):
        dis_espec_interp[:, idx] = np.interp(des_t_stamp, dis_t_stamp, dis_espec[:, idx])
    
    # Interpolate EDP data
    # An Nx3 array, ordered as (Ex, Ey, Ez)
    nComps = np.size(edp_e, 1)
    edp_e_interp = np.zeros([nTimes, nComps])
    for idx in range(nComps):
        edp_e_interp[:,idx] = np.interp(des_t_stamp, edp_t_stamp, edp_e[:,idx])
    edp_scpot_interp = np.interp(des_t_stamp, edp_t_stamp, edp_v)
        
    # Interpolate EDI data
    edi_cts1_0_interp = np.interp(des_t_stamp, edi_t_stamp, edi_cts1_0)
    edi_cts1_180_interp = np.interp(des_t_stamp, edi_t_stamp, edi_cts1_180)
    
    # Print results
    print('Time:                   ', np.shape(des_t), des_t.dtype)
    print('DES Density:            ', np.shape(des_n), des_n.dtype)
    print('DES Velocity:           ', np.shape(des_v), des_v.dtype)
    print('DES Temperature (para): ', np.shape(des_temp_para), des_temp_para.dtype)
    print('DES Temperature (perp): ', np.shape(des_temp_perp), des_temp_perp.dtype)
    print('FGM Magnetic Field:     ', np.shape(fgm_b_interp), fgm_b_interp.dtype)
    print('DIS Density:            ', np.shape(dis_n_interp), dis_n_interp.dtype)
    print('DIS Velocity:           ', np.shape(dis_v_interp), dis_v_interp.dtype)
    print('DIS Temperature (para): ', np.shape(dis_temp_para_interp), dis_temp_para_interp.dtype)
    print('DIS Temperature (perp): ', np.shape(dis_temp_perp_interp), dis_temp_perp_interp.dtype)
    
    ## Write a CSV file
    # Open file and write data
    data = {
        'Time'   :  des_t,
        'DES N'  : des_n,
        'DES Vx' : des_v[:,0],
        'DES Vy' : des_v[:,1],
        'DES Vz' : des_v[:,2],
        'DES T_para' : des_temp_para,
        'DES T_perp' : des_temp_perp,
        'FGM Bx' : fgm_b_interp[:,0],
        'FGM By' : fgm_b_interp[:,1],
        'FGM Bz' : fgm_b_interp[:,2],
        'FGM Bt' : fgm_b_interp[:,3],
        'FGM Clock_angle'  : fgm_clock_angle_interp, # Needed?
        'FGM Normal_angle' : fgm_normal_angle_interp, # Needed?
        'DIS N'  : dis_n_interp,
        'DIS Vx' : dis_v_interp[:,0],
        'DIS Vy' : dis_v_interp[:,1],
        'DIS Vz' : dis_v_interp[:,2],
        'DIS T_para' : dis_temp_para_interp,
        'DIS T_perp' : dis_temp_perp_interp,
        'DIS Temp'   : dis_temp_interp, # Needed?
        'EDP Ex' : edp_e_interp[:,0],
        'EDP Ey' : edp_e_interp[:,1],
        'EDP Ez' : edp_e_interp[:,2],
        'EDP Scpot'  : edp_scpot_interp, # Needed?
        'EDI cts1_0' : edi_cts1_0_interp,
        'EDI cts1_180'     : edi_cts1_180_interp 
    }
    
    # Add des_espec data en masse
    for col in range(np.size(des_espec, 1)):
        data['DES ESpec_{0:02d}'.format(col)] = des_espec[:,col]
    
    # Add dis_espec_interp data en masse
    for col in range(np.size(dis_espec_interp, 1)):
        data['DIS ESpec_{0:02d}'.format(col)] = dis_espec_interp[:,col]
    
    # Create a data frame
    data = pd.DataFrame(data, columns=data.keys())
    
    return data

# Read a row of the dataframe and convert it to an SQL-friendly string format
def getRowVals(rownum, data, spacecraft):
    rowvals = "'" + str(data.loc[[rownum]].values.tolist()[0][0]) + "',"
    rowvals += "'" + spacecraft + "', "
    rowvals += ', '.join([ str(f) for f in data.loc[[rownum]].values.tolist()[0][1:]]);
    return rowvals

# Create a table for the SQL DB. This will delete any existing table since this script should only be run once                                                         
def createTable(spacecraft, level, table_name, data):
    columns = list(data)
    
    # Check if table exists in DB already
    drop_table = 'DROP TABLE if exists mms1'
    c.execute(drop_table)
    
    # Create a new table with columns from the dataframe  
    create_table = 'CREATE TABLE ' +  table_name + ' ('
    
    # Create string containing column names and types
    colnames = 'Time TIMESTAMP, Spacecraft STRING , '
    for colname in columns[1:len(columns)-1]:
        colnames += colname.replace(' ', '_') + ' REAL, '
    colnames += columns[len(columns)-1].replace(' ', '_') + ' REAL'
    create_table += colnames + ');'
    
    c.execute(create_table)
    
    # Set indices
    set_primary_index = 'CREATE UNIQUE INDEX Time ON {}(Time)'.format(table_name)
    set_secondary_index = 'CREATE INDEX Spacecraft ON {}(Spacecraft)'.format(table_name)
    c.execute(set_primary_index)
    c.execute(set_secondary_index)

# Insert rows into the DB based on the output of getRowVals
def insertRows(spacecraft, data):
    # Insert rows of dataframe to table
    for rownum in range(1,len(data)):
        insert_row = ('INSERT INTO mms1 VALUES({});'.format(getRowVals(rownum, data, spacecraft)))
        c.execute(insert_row)

# Create a table and insert rows based on the data downloaded from data_export
def run(spacecraft, level, start_date, end_date):
    start_datetime = datetime.strptime(start_date, '%Y-%m-%dT%H:%M:%S')
    end_datetime = datetime.strptime(end_date, '%Y-%m-%dT%H:%M:%S')
    i=0

    while start_datetime < end_datetime:
        end_datetime = start_datetime + timedelta(days=1)
        data = data_export(spacecraft, level, start_datetime, end_datetime)

        if i == 0: 
            createTable(spacecraft, level, 'mms1', data)

        insertRows('mms1', data)
        start_datetime + timedelta(days=1) 
        i+=1

    connection.commit()
    connection.close()
    
# Open connection to SQLite DB
#sqlite_file = '/home/{}/pymms/sql/data.db'.format(user)
sqlite_file = '/Users/colinrsmall/data/sql/data.db'
connection = sqlite3.connect(sqlite_file)
c = connection.cursor()

spacecraft = 'mms1'
level = 'l2'
start_date = '2015-12-06T00:00:00'
end_date = '2015-12-06T23:59:59'

run(spacecraft, level, start_date, end_date)


import pathlib
import re
import csv
import datetime as dt
from . import mrmms_sdc_api as sdc
from cdflib import cdfread, epochs
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

tai_1958 = epochs.CDFepoch.compute_tt2000([1958, 1, 1, 0, 0, 0, 0, 0, 0])

class BurstSegment:
    def __init__(self, tstart, tstop, fom, discussion,
                 sourceid=None, createtime=None):
        '''
        Create an object representing a burst data segment.

        Parameters
        ----------
        fom : int, float
            Figure of merit given to the selection
        tstart : int, str, `datetime.datetime`
            The start time of the burst segment, given as a `datetime` object,
            a TAI time in seconds since 1 Jan. 1958, or as a string formatted
            as `yyyy-MM-dd hh:mm:SS`. Times are converted to `datetimes`.
        tstop : int, str, `datetime.datetime`
            The stop time of the burst segment, similar to `tstart`.
        discussion : str
            Description of the segment provided by the SITL
        sourceid : str
            Username of the SITL that made the selection
        file : str
            Name of the file containing the selection
        '''

        # Convert tstart to datetime
        if isinstance(tstart, str):
            tstart = dt.datetime.strptime(tstart, '%Y-%m-%d %H:%M:%S')
        elif isinstance(tstart, int):
            tstart = self.__class__.tai_to_datetime(tstart)

        # Convert tstop to datetime
        if isinstance(tstop, str):
            tstop = dt.datetime.strptime(tstop, '%Y-%m-%d %H:%M:%S')
        elif isinstance(tstop, int):
            tstop = self.__class__.tai_to_datetime(tstop)

        self.discussion = discussion
        self.createtime = createtime
        self.fom = fom
        self.sourceid = sourceid
        self.tstart = tstart
        self.tstop = tstop

    def __str__(self):
        return ('{0}   {1}   {2:3.0f}   {3}'
                .format(self.tstart, self.tstop, self.fom, self.discussion)
                )

    def __repr__(self):
        return ('selections.BurstSegment({0}, {1}, {2:3.0f}, {3})'
                .format(self.tstart, self.tstop, self.fom, self.discussion)
                )

    @staticmethod
    def datetime_to_list(t):
        return [t.year, t.month, t.day,
                t.hour, t.minute, t.second,
                t.microsecond // 1000, t.microsecond % 1000, 0
                ]

    @classmethod
    def datetime_to_tai(cls, t):
        t_list = cls.datetime_to_list(t)
        return int((epochs.CDFepoch.compute_tt2000(t_list) - tai_1958) // 1e9)

    @classmethod
    def tai_to_datetime(cls, t):
        tepoch  = epochs.CDFepoch()
        return tepoch.to_datetime(t * int(1e9) + tai_1958)

    @property
    def start_time(self):
        return self.tstart.strftime('%Y-%m-%d %H:%M:%S')

    @property
    def stop_time(self):
        return self.tstop.strftime('%Y-%m-%d %H:%M:%S')

    @property
    def taistarttime(self):
        return self.__class__.datetime_to_tai(self.tstart)

    @property
    def taiendtime(self):
        return self.__class__.datetime_to_tai(self.tstop)


def _burst_data_segments_to_burst_segment(data):
    '''
    Turn selections created by `MrMMS_SDC_API.burst_data_segements` and turn
    them into `BurstSegment` instances.

    Parameters
    ----------
    data : dict
        Data associated with each burst segment

    Returns
    -------
    result : list of `BurstSegment`
        Data converted to `BurstSegment` instances
    '''
    # Look at createtime and finishtime keys to see if either can
    # substitute for a file name time stamp
    
    result = []
    for tstart, tend, fom, discussion, sourceid, createtime, status in \
        zip(data['tstart'], data['tstop'], data['fom'],
            data['discussion'], data['sourceid'], data['createtime'],
            data['status']
            ):
        segment = BurstSegment(tstart, tend, fom, discussion,
                               sourceid=sourceid,
                               createtime=createtime)
        segment.status = status
        result.append(segment)
    return result


def _get_selections(type, start, stop,
                    sort=False, combine=False, latest=True, unique=False,
                    metadata=False, filter=None, case_sensitive=False):

    if latest and unique:
        raise ValueError('latest and unique keywords '
                         'are mutually exclusive.')

    # Burst selections can be made multiple times within the orbit.
    # Multiple submissions will be appear as repeated entires with
    # different create times, but only the last submission is the
    # official submission. To ensure that the last submission is
    # returned, look for selections submitted through the following
    # orbit as well.
    orbit_start = sdc.time_to_orbit(start)
    orbit_stop = sdc.time_to_orbit(stop) + 1

    # Get the selections
    data = sdc.burst_selections(type, orbit_start, orbit_stop)

    # Turn the data into BurstSegments. Adjacent segments returned
    # by `sdc.sitl_selections` have a 0 second gap between stop and
    # start times. Those returned by `sdc.burst_data_segments` are
    # separated by 10 seconds.
    if type in ('abs', 'sitl', 'gls', 'mp-dl-unh'):
        delta_t = 0.0
        converter = _sitl_selections_to_burst_segment
    elif type == 'sitl+back':
        delta_t = 10.0
        converter = _burst_data_segments_to_burst_segment
    else:
        raise ValueError('Invalid selections type {}'.format(type))

    # Convert data into BurstSegments
    # If there were no selections made, data will be empty without
    # keys (and throws error in _sitl_selections_to_burst_segment).
    try:
        data = converter(data)
    except KeyError:
        return []


    # Get metadata associated with orbit, sroi, and metadata
    if metadata:
        _get_segment_data(data, orbit_start, orbit_stop)

    # The official selections are those from the last submission
    # containing selections within the science_roi. Get rid of
    # all submissions except the last.
    if latest:
        data = _latest_segments(data, orbit_start, orbit_stop,
                                sitl=(type == 'sitl+back'))

    # Get rid of extra selections obtained by changing the
    # start and stop interval
    data = [segment
            for segment in data
            if (segment.tstart >= start) \
            and (segment.tstop <= stop)]

    # Additional handling of data
    if combine:
        combine_segments(data, delta_t)
    if sort:
        data = sort_segments(data)
    if unique:
        data = remove_duplicate_segments(data)
    if filter is not None:
        data = filter_segments(data, filter,
                               case_sensitive=case_sensitive)

    return data


def _get_segment_data(data, orbit_start, orbit_stop, sc='mms1'):
    '''
    Add metadata associated with the orbit, SROI, and SITL window
    to each burst segment.

    Parameters
    ----------
    data : list of BurstSegments
        Burst selections. Selections are altered in-place to
        have new attributes:
        Attribute            Description
        ==================   ======================
        orbit                Orbit number
        orbit_tstart         Orbit start time
        orbit_tstop          Orbit stop time
        sroi                 SROI number
        sroi_tstart          SROI start time
        sroi_tstop           SROI stop time
        sitl_window_tstart   SITL window start time
        sitl_window_tstop    SITL window stop time
        ==================   ======================
    orbit_start, orbit_stop : int or np.integer
        Orbit interval in which selections were made
    sc : str
        Spacecraft identifier
    '''
    idx = 0
    for iorbit in range(orbit_start, orbit_stop+1):
        # The start and end times of the sub-regions of interest.
        # These are the times in which selections can be made for
        # any given orbit.
        orbit = sdc.mission_events('orbit', iorbit, iorbit, sc=sc)
        sroi = sdc.mission_events('sroi', iorbit, iorbit, sc=sc)
        window = sdc.mission_events('sitl_window', iorbit, iorbit)
        tstart = min(sroi['tstart'])
        tend = max(sroi['tend'])

        # Find the burst segments that were selected within the
        # current SROI

        while idx < len(data):
            segment = data[idx]

            # Filter out selections from the previous orbit(s)
            # Stop when we get to the next orbit
            if segment.tstop < tstart:
                idx += 1
                continue
            if segment.tstart > tend:
                break

            # Keep segments from the same submission (create time).
            # If there is a new submission within the orbit, take
            # selections from the new submission and discard those
            # from the old.
            segment.orbit = iorbit
            segment.orbit_tstart = orbit['tstart'][0]
            segment.orbit_tstop = orbit['tend'][0]

            sroi_data = _get_sroi_number(sroi, segment.tstart, segment.tstop)
            segment.sroi = sroi_data[0]
            segment.sroi_tstart = sroi_data[1]
            segment.sroi_tstop = sroi_data[2]

            # The SITL Window is not defined as of orbit 1098, when
            # a SITL-defined window was implemented
            if iorbit < 1098:
                segment.sitl_window_tstart = window['tstart']
                segment.sitl_window_tstop = window['tend']

            idx += 1


def _get_sroi_number(sroi, tstart, tstop):
    '''
    Determine which sub-region of interest (SROI) in which a given
    time interval resides.

    Parameters
    ----------
    sroi : dict
        SROI information for a specific orbit
    tstart, tstop : `datetime.datetime`
        Time interval

    Returns
    -------
    result : tuple
       The SROI number and start and stop times.
    '''
    sroi_num = 0
    for sroi_tstart, sroi_tstop in zip(sroi['tstart'], sroi['tend']):
        sroi_num += 1
        if (tstart >= sroi_tstart) and (tstop <= sroi_tstop):
            break

    return sroi_num, sroi_tstart, sroi_tstop


def _latest_segments(data, orbit_start, orbit_stop, sitl=False):
    '''
    Return the latest burst selections submission from each orbit.

    Burst selections can be submitted multiple times but only
    the latest file serves as the official selections file.

    For the SITL, the SITL makes selections within the SITL
    window. If data is downlinked after the SITL window closes,
    selections on that data are submitted separately into the
    back structure and should be appended to the last submission
    that took place within the SITL window.

    Parameters
    ----------
    data : list of BurstSegments
        Burst segment selections
    orbit_start, orbit_stop : int or np.integer
        Orbits in which selections were made
    sitl : bool
        If true, the burst selections were made by the SITL and
        there are back structure submissions to take into
        consideration

    Returns
    -------
    results : list of BurstSegments
        The latest burst selections
    '''
    result = []
    for orbit in range(orbit_start, orbit_stop+1):
        # The start and end times of the sub-regions of interest.
        # These are the times in which selections can be made for
        # any given orbit.
        
        # SROI information is not available before orbit 239.
        # The first SROI is defined on 2015-11-06, which is only a
        # couple of months after formal science data collection began
        # on 2015-09-01.
        
        # However, similar information is available starting at 
        # 2015-08-10 (orbit 151) in the science_roi event type. It's
        # nearly equivalent because there was only a single SROI per
        # orbit during the earlier mission phases, but science_roi is
        # the span across all spacecraft.
        if orbit < 239:
            sroi = sdc.mission_events('science_roi', orbit, orbit)
        else:
            sroi = sdc.mission_events('sroi', orbit, orbit)
        tstart = min(sroi['tstart'])
        tend = max(sroi['tend'])

        # The SITL Window is not defined as of orbit 1098, when
        # a SITL-defined window was implemented
        if orbit >= 1098:
            sitl = False

        # Need to know when the SITL window closes in order to
        # keep submissions to the back structure.
        if sitl:
            sitl_window = sdc.mission_events('sitl_window', orbit, orbit)

        # Find the burst segments that were selected within the
        # current SROI
        create_time = None
        orbit_segments = []
        for idx, segment in enumerate(data):
            # Filter out selections from the previous orbit(s)
            # Stop when we get to the next orbit
            if segment.tstop < tstart:
                continue
            if segment.tstart > tend:
                break

            # Initialize with the first segment within the orbit.
            # Create times are the same for all selections within
            # a single submission. There may be several submissions
            # per orbit.
            if create_time is None:
                create_time = segment.createtime

            # Keep segments from the same submission (create time).
            # If there is a new submission within the orbit, take
            # selections from the new submission and discard those
            # from the old.
            #
            # Submissions to the back structure occur after the
            # SITL window has closed and are in addition to whatever
            # the latest submission was.
            #
            # GLS and ABS selections can occur after the SITL window
            # closes, but those are treated the same as selections
            # made within the SITL window.
            if abs(create_time - segment.createtime) < dt.timedelta(seconds=10):
                orbit_segments.append(segment)
            elif segment.createtime > create_time:
                if sitl and (segment.createtime > sitl_window['tend'][0]):
                    orbit_segments.append(segment)
                else:
                    create_time = segment.createtime
                    orbit_segments = [segment]
            else:
                continue

        # Truncate the segments and append this orbit's submissions
        # to the overall results.
        data = data[idx:]
        result.extend(orbit_segments)

    return result


def _mission_events_to_burst_segment(data):
    '''
    Turn selections created by `MrMMS_SDC_API.mission_events` and turn
    them into `BurstSegment` instances.

    Parameters
    ----------
    data : dict
        Data associated with each burst segment

    Returns
    -------
    result : list of `BurstSegment`
        Data converted to `BurstSegment` instances
    '''
    raise NotImplementedError


def _sitl_selections_to_burst_segment(data):
    '''
    Turn selections created by `MrMMS_SDC_API.sitl_selections` and turn
    them into `BurstSegment` instances.

    Parameters
    ----------
    data : dict
        Data associated with each burst segment

    Returns
    -------
    result : list of `BurstSegment`
        Data converted to `BurstSegment` instances
    '''
    result = []
    for idx in range(len(data['fom'])):
        result.append(BurstSegment(data['tstart'][idx], data['tstop'][idx],
                                   data['fom'][idx], data['discussion'][idx],
                                   sourceid=data['sourceid'][idx],
                                   createtime=data['createtime'][idx],
                                   )
                      )
    return result


def combine_segments(data, dt_contig=0):
    '''
    Combine contiguous burst selections into single selections.

    Parameters
    ----------
    data : list of `BurstSegment`
        Selections to be combined.
    dt_contig : int
        Time interval between adjacent selections. For selections
        returned by `pymms.sitl_selections()`, this is 0. For selections
        returned by `pymms.burst_data_segment()`, this is 10.
    '''
    # Any time delta > dt_contig sec indicates the end of a contiguous interval
    t_deltas = [(seg1.tstart - seg0.tstop).total_seconds()
                for seg1, seg0 in zip(data[1:], data[:-1])
                ]
    
    # Time deltas has one fewer element than data at this stage. Append
    # infinity to the time deltas to indicate that the last element in data
    # does not have a contiguous neighbor. This will make the number of
    # elements in each array equal UNLESS data is empty. Do not append if
    # data is empty (to avoid indexing errors below).
    if len(data) > 0:
        t_deltas.append(1000)
    
    icontig = 0  # Current contiguous interval
    result = []

    # Check if adjacent elements are continuous in time
    #   - Use itertools.islice to select start index without copying array
    for idx, t_delta in enumerate(t_deltas):
        # Contiguous segments are separated by dt_contig seconds.
        # The last element of t_delta = 1000, so not pass this
        # condition and idx+1 will not cause an IndexError
        if t_delta == dt_contig:
            # And unique segments have the same fom and discussion
            if (data[icontig].fom == data[idx+1].fom) and \
                    (data[icontig].discussion == data[idx+1].discussion):
                continue

        # End of a contiguous interval
        data[icontig].tstop = data[idx].tstop

        # Next interval
        icontig = icontig + 1

        # Move data for new contiguous segments to beginning of array
        try:
            data[icontig] = data[idx+1]
        except IndexError:
            pass

    # Truncate data beyond last contiguous interval
    del data[icontig:]


def filter_segments(data, filter, case_sensitive=False):
    '''
    Filter burst selections by their discussion string.

    Parameters
    ----------
    data : dict
        Selections to be combined. Must have key 'discussion'.
    filter : str
        Regular expression used to filter the data
    case_sensitive : bool
        Make the filter case sensitive
    '''
    # Make case-insensitive searches the default
    flags = re.IGNORECASE
    if case_sensitive:
        flags = 0
    
    return [seg
            for seg in data
            if re.search(filter, seg.discussion, flags)]


def plot_metric(ref_data, test_data, fig, labels, location,
                nbins=10):
    '''
    Visualize the overlap between segments.

    Parameters
    ----------
    ref_data : list of `BurstSegment`s
        Reference burst segments
    test_data : list of `BurstSegment`s
        Test burst segments. Determine which test segments
        overlap with the reference segments and by how much
    labels : tuple of str
        Labels for the reference and test segments
    location : tuple
        Location of the figure (row, col, nrows, ncols)
    nbins : int
        Number of histogram bins to create

    Returns:
    --------
    ax : `matplotlib.pyplot.Axes`
        Axes in which data is displayed
    ref_test_data : list of `BurstSegment`s
        Reference data that falls within the [start, stop] times
        of the test data.
    '''

    # Determine by how much the test data overlaps with
    # the reference data.
    ref_test = []
    ref_test_data = []
    ref_test = [selection_overlap(segment, test_data)
                for segment in ref_data]

    # Overlap statistics
    #   - Number of segments selected
    #   - Percentage of segments selected
    #   - Percent overlap from each segment
    ref_test_selected = sum(selection['n_selections'] > 0
                            for selection in ref_test)
    ref_test_pct_selected = ref_test_selected / len(ref_test) * 100.0
    ref_test_pct_overlap = [selection['pct_overlap'] for selection in ref_test]

    # Calculate the plot index from the (row,col) subplot location
    plot_idx = lambda rowcol, ncols : (rowcol[0]-1)*ncols + rowcol[1]

    # Create a figure
    ax = fig.add_subplot(location[2], location[3],
                         plot_idx(location[0:2], location[3]))
    hh = ax.hist(ref_test_pct_overlap, bins=nbins, range=(0, 100))
    #ax.set_xlabel('% Overlap Between {0} and {1} Segments'.format(*labels))
    if location[0] == location[2]:
        ax.set_xlabel('% Overlap per Segment')
    if location[1] == 1:
        ax.set_ylabel('Occurrence')
    ax.text(0.5, 0.98, '{0:4.1f}% of {1:d}'
              .format(ref_test_pct_selected, len(ref_test)),
              verticalalignment='top', horizontalalignment='center',
              transform=ax.transAxes)
    ax.set_title('{0} Segments\nSelected by {1}'.format(*labels))

    return ax, ref_test


def metric(sroi=None, output_dir=None, figtype=None):
    do_sroi=False
    if sroi in (1, 2, 3):
        do_sroi=True

    if output_dir is None:
        output_dir = pathlib.Path('~/').expanduser()
    else:
        output_dir = pathlib.Path(output_dir).expanduser()

    starttime = dt.datetime(2019, 10, 17)

    # Find SROI
    #start_date, end_date = gls_get_sroi(starttime)
    start_date = dt.datetime(2019, 10, 19)
    #end_date = start_date + dt.timedelta(days=5)
    end_date = dt.datetime.now()

    abs_data = selections('abs', start_date, end_date,
                          latest=True, combine=True, metadata=do_sroi)

    gls_data = selections('mp-dl-unh', start_date, end_date,
                          latest=True, combine=True, metadata=do_sroi)

    sitl_data = selections('sitl+back', start_date, end_date,
                           latest=True, combine=True, metadata=do_sroi)

    # Filter by SROI
    if do_sroi:
        abs_data = [s for s in abs_data if s.sroi == sroi]
        sitl_data = [s for s in sitl_data if s.sroi == sroi]
        gls_data = [s for s in gls_data if s.sroi == sroi]

    sitl_mp_data = filter_segments(sitl_data, '(MP|Magnetopause)')

    # Create a figure
    nbins = 10
    nrows = 4
    ncols = 3
    fig = plt.figure(figsize=(8.5, 10))
    fig.subplots_adjust(hspace=0.55, wspace=0.3)

    # GLS-SITL Comparison
    ax, gls_sitl = plot_metric(gls_data, sitl_data, fig,
                               ('GLS', 'SITL'), (1, 1, nrows, ncols),
                               nbins=nbins)
    ax, sitl_gls = plot_metric(sitl_data, gls_data, fig,
                               ('SITL', 'GLS'), (2, 1, nrows, ncols),
                               nbins=nbins)
    ax, gls_sitl_mp = plot_metric(gls_data, sitl_mp_data, fig,
                                  ('GLS', 'SITL MP'), (3, 1, nrows, ncols),
                                  nbins=nbins)
    ax, sitl_mp_gls = plot_metric(sitl_mp_data, gls_data, fig,
                                  ('SITL MP', 'GLS'), (4, 1, nrows, ncols),
                                  nbins=nbins)

    # ABS-SITL Comparison
    ax, abs_sitl = plot_metric(abs_data, sitl_data, fig,
                               ('ABS', 'SITL'), (1, 2, nrows, ncols),
                               nbins=nbins)
    ax, sitl_abs = plot_metric(sitl_data, abs_data, fig,
                               ('SITL', 'ABS'), (2, 2, nrows, ncols),
                               nbins=nbins)
    ax, abs_sitl_mp = plot_metric(abs_data, sitl_mp_data, fig,
                                  ('ABS', 'SITL MP'), (3, 2, nrows, ncols),
                                  nbins=nbins)
    ax, sitl_mp_abs = plot_metric(sitl_mp_data, abs_data, fig,
                                  ('SITL MP', 'ABS'), (4, 2, nrows, ncols),
                                  nbins=nbins)

    # GLS-ABS Comparison
    abs_mp_data = [abs_data[idx]
                   for idx, s in enumerate(abs_sitl_mp)
                   if s['n_selections'] > 0]

    ax, gls_abs = plot_metric(gls_data, abs_data, fig,
                              ('GLS', 'ABS'), (1, 3, nrows, ncols),
                              nbins=nbins)
    ax, abs_gls = plot_metric(abs_data, gls_data, fig,
                              ('ABS', 'GLS'), (2, 3, nrows, ncols),
                              nbins=nbins)
    ax, gls_abs_mp = plot_metric(gls_data, abs_mp_data, fig,
                                 ('GLS', 'ABS MP'), (3, 3, nrows, ncols),
                                 nbins=nbins)
    ax, abs_mp_gls = plot_metric(abs_mp_data, gls_data, fig,
                                 ('ABS MP', 'GLS'), (4, 3, nrows, ncols),
                                 nbins=nbins)

    # Save the figure
    if figtype is not None:
        sroi_str = ''
        if do_sroi:
            sroi_str = '_sroi{0:d}'.format(sroi)
        filename = (output_dir
                    / '_'.join(('selections_metric' + sroi_str,
                                start_date.strftime('%Y%m%d%H%M%S'),
                                end_date.strftime('%Y%m%d%H%M%S')
                                )))
        filename = filename.with_suffix('.' + figtype)
        plt.savefig(filename.expanduser())

    plt.show()


def print_segments(data, full=False):
    '''
    Print details of the burst selections.

    Parameters
    ----------
    data : `BurstSegment` or list of `BurstSegment`
        Selections to be printed. Must have keys 'tstart', 'tstop',
        'fom', 'sourceid', and 'discussion'
    '''
    if full:
        source_len = max(len(s.sourceid) for s in data)
        source_len = max(source_len, 8)
        header_fmt = '{0:>19}   {1:>19}   {2:>19}   {3:>5}   ' \
                     '{4:>19}   {5:>'+str(source_len)+'}   {6}'
        data_fmt = '{0:>19}   {1:>19}   {2:>19}   {3:5.1f}   ' \
                   '{4:>19}, {5:>'+str(source_len)+'}   {6}'
        print(header_fmt.format('TSTART', 'TSTOP', 'CREATETIME',
                                'FOM', 'STATUS', 'SOURCEID', 'DISCUSSION'
                                )
              )
        for s in data:
            try:
                status = s.status
            except AttributeError:
                status = ''
            
            createtime = dt.datetime.strftime(s.createtime,
                                              '%Y-%m-%d %H:%M:%S')
            print(data_fmt.format(s.start_time, s.stop_time,
                                  createtime, s.fom, status, s.sourceid,
                                  s.discussion)
                  )
        return


    print('{0:>19}   {1:>19}   {2}   {3}'
          .format('TSTART', 'TSTOP', 'FOM', 'DISCUSSION')
          )

    if isinstance(data, list):
        for selection in data:
            print(selection)
    else:
        print(data)


def read_csv(filename, start_time=None, stop_time=None, header=True):
    '''
    Read a CSV file with burst segment selections.

    Parameters
    ----------
    filename : str
        The name of the file to which `data` is to be read
    start_time : str or `datetime.datetime`
        Filter results to contain segments selected on or
        after this time. Possible only if `header` is True
        and if a column is named `'start_time'`
    stop_time : str or `datetime.datetime`
        Filter results to contain segments selected on or
        before this time. Possible only if `header` is True
        and if a column is named `'stop_time'`
    header : bool
        If `True`, the csv file has a header indicating the
        names of each column. If `header` is `False`, the
        assumed column names are 'start_time', 'stop_time',
        'fom', 'sourceid', 'discussion', 'createtime'.

    Returns
    -------
    data : list of `BurstSegment`
        Burst segments read from the csv file
    '''
    # Convert time itnerval to datetimes if needed
    if isinstance(start_time, str):
        start_time = dt.datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S')
    if isinstance(stop_time, str):
        stop_time = dt.datetime.strptime(stop_time, '%Y-%m-%d %H:%M:%S')

    file = pathlib.Path(filename)
    data = []

    # Read the file
    with open(file.expanduser(), 'r', newline='') as csvfile:
        csvreader = csv.reader(csvfile)

        # Take column names from file header
        if header:
            keys = next(csvreader)

        # Read the rows
        for row in csvreader:
            # Select the data within the time interval
            if start_time is not None:
                tstart = dt.datetime.strptime(row[0],
                                              '%Y-%m-%d %H:%M:%S')
                if tstart < start_time:
                    continue
            if stop_time is not None:
                tstop = dt.datetime.strptime(row[1],
                                             '%Y-%m-%d %H:%M:%S')
                if tstop > stop_time:
                    continue  # BREAK if sorted!!

            # Initialize segment with required fields then add
            # additional fields after
            data.append(BurstSegment(row[0], row[1], float(row[2]), row[4],
                                     sourceid=row[3], createtime=row[5]
                                     )
                        )

    return data


def remove_duplicate_segments(data):
    '''
    If SITL or GLS selections are submitted multiple times,
    there can be multiple copies of the same selection, or
    altered selections that overlap with what was selected
    previously. Find overlapping segments and select those
    from the most recent file, as indicated by the file name.

    Parameters
    ----------
    data : list of `BurstSegment`
        Selections from which to prude duplicates. Segments
        must be sorted by `tstart`.

    Returns
    -------
    results : list of `BurstSegments`
        Unique burst segments.
    '''
    results = data.copy()
    overwrite_times = set()
    for idx, segment in enumerate(data):
        iahead = idx + 1

        # Segments should already be sorted. Future segments
        # overlap with current segment if the future segement
        # start time is closer to the current segment start
        # time than is the current segment's end time.
        while ((iahead < len(data)) and
               ((data[iahead].tstart - segment.tstart) <
                (segment.tstop - segment.tstart))
               ):

            # Remove the segment with the earlier create time
            if segment.createtime < data[iahead].createtime:
                remove_segment = segment
                overwrite_time = segment.createtime
            else:
                remove_segment = data[iahead]
                overwrite_time = data[iahead].createtime

            # The segment may have already been removed if
            # there are more than one other segments that
            # overlap with it.
            try:
                results.remove(remove_segment)
                overwrite_times.add(overwrite_time)
            except ValueError:
                pass

            iahead += 1

    # Remove all segments with create times in the overwrite set
    # Note that the model results do not appear to be reproducible
    # so that every time the model is run, a different set of
    # selections are created. Using the filter below, data from
    # whole days can be removed if two processes create overlapping
    # segments at the edges of their time intervals.
#     results = [segment
#                for segment in results
#                if segment.createtime not in overwrite_times
#                ]

    print('# Segments Removed: {}'.format(len(data) - len(results)))
    return results


def remove_duplicate_segments_v1(data):
    idx = 0  # current index
    i = idx  # look-ahead index
    noverlap = 0
    result = []
    for idx in range(len(data)):
        if idx < i:
            continue
        print(idx)

        # Reference segment is the current segment
        ref_seg = data[idx]
        t0_ref = ref_seg.taistarttime
        t1_ref = ref_seg.taiendtime
        dt_ref = t1_ref - t0_ref

        if idx == len(data) - 2:
            import pdb
            pdb.set_trace()

        try:
            # Test segment are after the reference segment. The
            # test segment overlaps with the reference segment
            # if its start time is closer to the reference start
            # time than is the reference end time. If there is
            # overlap, keep the segment that was created more
            # recently.
            i = idx + 1
            while ((data[i].taistarttime - t0_ref) < dt_ref):
                noverlap += 1
                test_seg = data[i]
                if data[i].createtime > data[idx].createtime:
                    ref_seg = test_seg
                i += 1
        except IndexError:
            pass

        result.append(ref_seg)

    print('# Overlapping Segments: {}'.format(noverlap))
    return result


def selection_overlap(ref, tests):
    '''
    Gather overlap statistics.
    
    Parameters
    ----------
    ref : `selections.BurstSegment`
        The reference burst segment.
    tests : list of `selections.BurstSegment`
        The burst segements against which the reference segment is compared.
    
    Returns
    -------
    out : dict
        Data regarding how much the reference segment overlaps with the
        test segments.
    '''
    out = {'dt': ref.tstop - ref.tstart,
           'dt_next': dt.timedelta(days=7000),
           'n_selections': 0,
           't_overlap': dt.timedelta(seconds=0.0),
           't_overselect': dt.timedelta(seconds=0.0),
           'pct_overlap': 0.0,
           'pct_overselect': 0.0
           }

    # Find which selections overlap with the given entry and by how much
    tdelta = dt.timedelta(days=7000)
    for test in tests:

        if ((test.tstart <= ref.tstop) and
            (test.tstop >= ref.tstart)
            ):
            out['n_selections'] += 1
            out['t_overlap'] += (min(test.tstop, ref.tstop)
                                 - max(test.tstart, ref.tstart)
                                 )

        # Time to nearest interval
        out['dt_next'] = min(out['dt_next'], abs(test.tstart - ref.tstart))

    # Overlap and over-selection statistics
    if out['n_selections'] > 0:
        out['t_overselect'] = out['dt'] - out['t_overlap']
        out['pct_overlap'] = out['t_overlap'] / out['dt'] * 100.0
        out['pct_overselect'] = out['t_overselect'] / out['dt'] * 100.0
    else:
        out['t_overselect'] = out['dt']
        out['pct_overselect'] = 100.0

    return out


def selections(type, start, stop,
               sort=False, combine=False, latest=True, unique=False,
               metadata=False, filter=None, case_sensitive=False):
    '''
    Factory function for burst data selections.

    Parameters
    ----------
    type : str
        Type of burst data selections to retrieve. Options are 'abs',
        'abs-all', 'sitl', 'sitl+back', 'gls', 'mp-dl-unh'.
    start, stop : `datetime.datetime`
        Interval over which to retrieve data
    sort : bool
        Sort burst segments by time. Submissions to the back structure and
        multiple submissions or process executions in one SITL window can
        cause multiples of the same selection or out-of-order selections.
    combine : bool
        Combine adjacent selection into one selection. Due to downlink
        limitations, long time duration burst segments must be broken into
        smaller chunks.
    latest : bool
        For each SROI, keep only those times with the most recent time
        of creation. Cannot be used with `unique`.
    metadata : bool
        Retrieve the orbit number and time interval, SROI number and time
        interval, and SITL window time interval.
    unique : bool
        Return only unique segments. See `sort`. Also, a SITL may adjust
        the time interval of their selections, so different submissions will
        have duplicates selections but with different time stamps. This is
        accounted for.
    filter : str
        Filter the burst segments by applying the regular expression to
        the segment's discussions string.

    Returns
    -------
    results : list of `BurstSegment`
        Each burst segment.
    '''
    return _get_selections(type, start, stop,
                           sort=sort, combine=combine, unique=unique,
                           latest=latest, metadata=metadata,
                           filter=filter, case_sensitive=case_sensitive)


def sort_segments(data, createtime=False):
    '''
    Sort abs, sitl, or gls selections into ascending order.

    Parameters
    ----------
    data : list of `BurstSegement`
        Selections to be sorted. Must have keys 'start_time', 'end_time',
        'fom', 'discussion', 'tstart', and 'tstop'.
    createtime: bool
        Sort by time created instead of by selection start time.

    Returns
    -------
    results : list of `BurstSegement`
        Inputs sorted by time
    '''
    if createtime:
        return sorted(data, key=lambda x: x.createtime)
    return sorted(data, key=lambda x: x.tstart)


def write_csv(filename, data, append=False):
    '''
    Write a CSV file with burst data segment selections.

    Parameters
    ----------
    filename : str
        The name of the file to which `data` is to be written
    data : list of `BurstSegment`
        Burst segments to be written to the csv file
    append : bool
        If True, `data` will be appended to the end of the file.
    '''
    header = ('start_time', 'stop_time', 'fom', 'sourceid',
              'discussion', 'createtime')

    mode = 'w'
    if append:
        mode = 'a'

    file = pathlib.Path(filename)
    with open(file.expanduser(), mode, newline='') as csvfile:
        csvwriter = csv.writer(csvfile)

        if not append:
            csvwriter.writerow(header)

        for segment in data:
            csvwriter.writerow([segment.start_time,
                                segment.stop_time,
                                segment.fom,
                                segment.sourceid,
                                segment.discussion,
                                segment.createtime
                                ]
                               )


if __name__ == 'main':
    from heliopy import config
    import pathlib

    # Inputs
    sc = sys.argv[0]
    start_date = sys.argv[1]
    if len(sys.argv) == 3:
        dir = sys.argv[2]
    else:
        dir = pathlib.Path(config['download_dir']) / 'figures' / 'mms'

    start_date = dt.datetime.strptime(start_date, '%Y-%m-%dT%H:%M:%S')

    # Plot the data
    fig = plot_context(sc, start_date, dir=dir)


#!/usr/bin/python
import requests
import urllib3

def parse_form(r):
    '''Parse key-value pairs from the log-in form
    
    Parameters
    ----------
    r (object):    requests.response object.
    
    Returns
    -------
    form (dict):   key-value pairs parsed from the form.
    '''
    # Find action URL
    pstart = r.text.find('<form')
    pend = r.text.find('>', pstart)
    paction = r.text.find('action', pstart, pend)
    pquote1 = r.text.find('"', pstart, pend)
    pquote2 = r.text.find('"', pquote1+1, pend)
    url5 = r.text[pquote1+1:pquote2]
    url5 = url5.replace('&#x3a;', ':')
    url5 = url5.replace('&#x2f;', '/')

    # Parse values from the form
    pinput = r.text.find('<input', pend+1)
    inputs = {}
    while pinput != -1:
        # Parse the name-value pair
        pend = r.text.find('/>', pinput)

        # Name
        pname = r.text.find('name', pinput, pend)
        pquote1 = r.text.find('"', pname, pend)
        pquote2 = r.text.find('"', pquote1+1, pend)
        name = r.text[pquote1+1:pquote2]

        # Value
        if pname != -1:
            pvalue = r.text.find('value', pquote2+1, pend)
            pquote1 = r.text.find('"', pvalue, pend)
            pquote2 = r.text.find('"', pquote1+1, pend)
            value = r.text[pquote1+1:pquote2]
            value = value.replace('&#x3a;', ':')

            # Extract the values
            inputs[name] = value

        # Next iteraction
        pinput = r.text.find('<input', pend+1)
    
    form = {'url': url5,
            'payload': inputs}
    
    return form


def login(username, password):
    '''Log-In to the MMS Science Data Center.
    
    Submit log-in credentials to the MMS Science Data Center team site.

    Parameters:
    -----------
    username (str):     Account username.
    password (str):     Account password.
    
    Returns:
    --------
    Cookies (dict):     Session cookies for continued access to the SDC.
    '''
    
    # Disable warnings because we are not going to obtain certificates
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    # Attempt to access the site
    #   - Each of the redirects are stored in the history attribute
    url0 = 'https://lasp.colorado.edu/mms/sdc/team/'
    r = requests.get(url0, verify=False)

    # Extract cookies and url
    cookies = r.cookies
    for response in r.history:
        cookies.update(response.cookies)
    
        try:
            url = response.headers['Location']
        except:
            pass
    
    # Submit login information
    payload = {'j_username': username, 'j_password': password}
    r = requests.post(url, cookies=cookies, data=payload, verify=False)

    # After submitting info, we land on a page with a form
    #   - Parse form and submit values to continue
    form = parse_form(r)
    r = requests.post(form['url'], cookies=cookies, data=form['payload'], verify=False)

    # Update cookies to get session information
    cookies = r.cookies
    for response in r.history:
        cookies.update(response.cookies)

    return cookies